// ===========================================================================
// ExControl.h
// custom control
//
//                                                                    kanegon
// ===========================================================================

#if !defined(_EXCONTROL_H_)
#define _EXCONTROL_H_

#include <commctrl.h>
#include "resource.h"


/////////////////////////////////////////////////////////////////////////////
// CExEdit

class CExEdit : public CWindowImpl<CExEdit>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, "EDIT")

public:
	void SetSel(int nStartChar, int nEndChar, BOOL bNoScroll = FALSE) {
		::SendMessage(m_hWnd, EM_SETSEL, nStartChar, nEndChar);
		if(!bNoScroll)
			::SendMessage(m_hWnd, EM_SCROLLCARET, 0, 0L);
	}
	void ReplaceSel(LPCTSTR lpszNewText, BOOL bCanUndo = FALSE) {
		::SendMessage(m_hWnd, EM_REPLACESEL, (WPARAM) bCanUndo, (LPARAM)lpszNewText);
	}
	void InsertText(int nInsertAfterChar, LPCTSTR lpstrText, BOOL bNoScroll = FALSE, BOOL bCanUndo = FALSE) {
		SetSel(nInsertAfterChar, nInsertAfterChar, bNoScroll);
		ReplaceSel(lpstrText, bCanUndo);
	}
	void AppendText(LPCTSTR lpstrText, BOOL bNoScroll = FALSE, BOOL bCanUndo = FALSE) {
		InsertText(GetWindowTextLength(), lpstrText, bNoScroll, bCanUndo);
	}
	void EmptyUndoBuffer() {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, EM_EMPTYUNDOBUFFER, 0, 0L);
	}

// Attributes
	BOOL CanUndo() const {
		ATLASSERT(::IsWindow(m_hWnd));
		return (BOOL)::SendMessage(m_hWnd, EM_CANUNDO, 0, 0L);
	}
	int GetLineCount() const {
		ATLASSERT(::IsWindow(m_hWnd));
		return (int)::SendMessage(m_hWnd, EM_GETLINECOUNT, 0, 0L);
	}
	BOOL GetModify() const {
		ATLASSERT(::IsWindow(m_hWnd));
		return (BOOL)::SendMessage(m_hWnd, EM_GETMODIFY, 0, 0L);
	}
	void SetModify(BOOL bModified = TRUE) {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, EM_SETMODIFY, bModified, 0L);
	}

	DWORD GetSel() const {
		ATLASSERT(::IsWindow(m_hWnd));
		return ::SendMessage(m_hWnd, EM_GETSEL, 0, 0L);
	}
	void GetSel(int& nStartChar, int& nEndChar) const {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, EM_GETSEL, (WPARAM)&nStartChar, (LPARAM)&nEndChar);
	}
//	int GetSelText(LPTSTR pszText) const {
//		ATLASSERT(::IsWindow(m_hWnd));
//		return ::SendMessage(m_hWnd, EM_GETSELTEXT, 0, (LPARAM)pszText);
//	}


// Clipboard
	BOOL Undo() {
		ATLASSERT(::IsWindow(m_hWnd));
		return (BOOL)::SendMessage(m_hWnd, EM_UNDO, 0, 0L);
	}
	void Clear() {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, WM_CLEAR, 0, 0L);
	}
	void Copy() {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, WM_COPY, 0, 0L);
	}
	void Cut() {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, WM_CUT, 0, 0L);
	}
	void Paste() {
		ATLASSERT(::IsWindow(m_hWnd));
		::SendMessage(m_hWnd, WM_PASTE, 0, 0L);
	}

public:
	BEGIN_MSG_MAP(CExEdit)
	END_MSG_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CExToolbar

class CExToolbar : public CWindowImpl<CExToolbar>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, TOOLBARCLASSNAME)

	HWND Create(HWND hwndParent, DWORD dwStyle, UINT nID, int nBitmaps, HINSTANCE hBMInst,
				UINT wBMID, LPCTBBUTTON lpButtons, int iNumButtons,
				int dxButton, int dyButton, int dxBitmap, int dyBitmap);

public:
	void SetStyle(DWORD dwStyle) {
		::SendMessage(m_hWnd, TB_SETSTYLE, 0, dwStyle);
	}

public:
	BEGIN_MSG_MAP(CExToolbar)
	MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
	END_MSG_MAP()

	LRESULT OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
};


/////////////////////////////////////////////////////////////////////////////
// CExStatusBar

class CExStatusBar : public CWindowImpl<CExStatusBar>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, STATUSCLASSNAME)

public:
	BOOL SetText(int nPane, LPCTSTR lpszText, int nType = 0) {
		return (BOOL)::SendMessage(m_hWnd, SB_SETTEXT, (nPane | nType), (LPARAM)lpszText);
	}

	BOOL SetParts(int nParts, int* pWidths) {
		return (BOOL)::SendMessage(m_hWnd, SB_SETPARTS, nParts, (LPARAM)pWidths);
	}

public:
	BEGIN_MSG_MAP(CExStatusBar)
	END_MSG_MAP()
};


#endif // _EXCONTROL_H_
